# -*- coding: utf8 -*-
"""PyTistory를 정의하는 모듈입니다.
"""
import os
import configparser
import multiprocessing
import webbrowser
import time

import requests

from .api import Blog, Category, Post, Comment, Guestbook
from .exceptions import (InvalidSectionError, InvalidNameError, ConfigurationError,
                         OptionNotFoundError, EmailAuthError, WebDriverError, TokenNotFoundError,
                         InvalidAccountError)
from .callback import CallbackServer

TISTORY_AUTHORIZE_URL = 'https://www.tistory.com/oauth/authorize'
TISTORY_AUTHORIZE_PARAMS = '?client_id={0}&redirect_uri=http://0.0.0.0:5000/callback&response_type=token'

CONFIG_SECTION_NAME = 'pytistory'
CONFIG_CLIENT_ID = 'client_id'
CONFIG_TISTORY_ID = 'tistory_id'
CONFIG_TISTORY_PASSWORD = 'tistory_password'

DEFAULT_CONFIG_FILE_NAME = '~/.pytistory/credentials.ini'


def callback_process(namespace, event):
    """flask가 실행되는 process 함수.

    Callback Server를 다른 프로세스로 만들어서 쓰기 위한 함수입니다.
    하나의 프로세스로 동작합니다.

    :param namespace: access_token을 주고 받기 위한 인자
    :type namespace: :class:`multiprocessing.managers.Namespace`
    :param event: 이벤트를 받기 위한 인자
    :type event: :class:`multiprocessing.Event`
    """
    callback = CallbackServer(namespace, event)
    callback.prepare()


class PyTistory:
    """Tistory Api 전체적으로 묶어주는 클래스입니다.

    다음과 같은 기능을 합니다.

      - 인증
    """
    # pylint: disable=too-many-instance-attributes,too-few-public-methods
    # 해당 클래스의 멤버를 통해 api를 부르므로, too-few-public-methods는 체크할 필요가 없다.

    def __init__(self):
        self.__TESTING__ = False

        self.client_id = None
        self.tistory_id = None
        self.tistory_password = None

        self.access_token = None

        self.blog = Blog()
        self.post = Post()
        self.category = Category()
        self.comment = Comment()
        self.guestbook = Guestbook()

    @staticmethod
    def _is_listening():
        try:
            requests.get('http://localhost:5000/')
        except requests.exceptions.ConnectionError:
            return False

        return True

    def _set_access_token(self, headless_auth):
        """Tistory의 access_token을 설정하는 함수입니다.

        `CallbackServer`를 구동시킨 후, 티스토리 인증 창을 브라우저를 통해 엽니다.
        그 후 인증이 되었다면, access_token을 받아옵니다.

        :param headless_auth: Headless 브라우저를 사용해서 인증을 시도할 때 이 인자를 사용합니다.
        :param headless_auth: bool
        :raises EmailAuthError: 티스토리 인증 중 이메일 인증이 필요한 경우입니다.
        :raises TokenNotFoundError: 만약 access_token이 정상적으로 받아와지지 않았을 경우입니다.
        """
        if headless_auth:
            session = requests.Session()
            resp = session.post('https://www.tistory.com/auth/login', data=dict({
                'redirectUrl': 'https://www.tistory.com/oauth/authorize?client_id={}&redirect_uri=http://0.0.0.0:5000/callback&response_type=token'.format(self.client_id),
                'loginId': self.tistory_id,
                'password': self.tistory_password,
                'rememberLoginId': 1
            }), allow_redirects=False)

            if resp.status_code != 302:
                raise ConfigurationError("Cannot Sign into Tistory")

            resp = session.get(
                'https://www.tistory.com/oauth/authorize?client_id={}&redirect_uri=http://0.0.0.0:5000/callback&response_type=token'.format(self.client_id), allow_redirects=False)

            if resp.status_code != 302 or 'Location' not in resp.headers:
                raise ConfigurationError(
                    "Cannot get access token. try configure with browser")

            location = resp.headers['Location']
            params = list(map(lambda x: x.split('='),
                              location.split('#')[1].split('&')))

            for param in params:
                if param[0] == 'access_token':
                    self.access_token = param[1]
                    break

            if self.access_token is None:
                raise ConfigurationError('Cannot get the access_token')

        else:
            multiprocessing_manager = multiprocessing.Manager()
            namespace = multiprocessing_manager.Namespace()
            event = multiprocessing.Event()

            process = multiprocessing.Process(
                target=callback_process, args=(namespace, event))
            process.start()

            if not self.__TESTING__:
                while not self._is_listening():
                    time.sleep(0.1)

                request_uri = TISTORY_AUTHORIZE_URL + \
                    TISTORY_AUTHORIZE_PARAMS.format(self.client_id)
                webbrowser.open_new(request_uri)

            event.wait()
            if hasattr(namespace, 'access_token') and namespace.access_token:  # pylint: disable=E1101
                self.access_token = namespace.access_token  # pylint: disable=E1101
                # disable the pylint message E1101 `Instance of 'Namespace'
                # has no 'access_token' member'`
                # checked in if statement
            else:
                raise TokenNotFoundError('Cannot get the access token from a server. :' +
                                         namespace.error_description if namespace.error_description is not None else 'We can`t find an error.\n please report this as an issue :)\nhttps://github.com/JeongUkJae/pytistory/issues')  # pylint: disable=E1101

            process.join()

    def _configure_with_file(self, file_name, no_error=False):
        """파일을 통한 설정

        :param file_name: 환경 설정파일이 담겨있는 파일 이름, defaults to None
        :type file_name: str, optional
        :param no_error: 에러를 일으키지 않는 옵션 (기본 설정 파일일 때 사용), defaults to False
        :type no_error: bool, optional
        """
        if not file_name:
            raise ConfigurationError("You have to pass a file name")

        if not os.path.exists(file_name) or not os.path.isfile(file_name):
            if no_error:
                return

            raise ConfigurationError(
                "You have to pass the file name that is valid.")

        config = configparser.ConfigParser()
        with open(file_name) as fp:
            config.read_string(fp.read())

            if CONFIG_SECTION_NAME not in config:
                if no_error:
                    return

                raise InvalidSectionError('Cannot find a `{}` section in `{}`.'.
                                          format(CONFIG_SECTION_NAME, file_name))
            if CONFIG_CLIENT_ID not in config[CONFIG_SECTION_NAME]:
                if no_error:
                    return

                raise InvalidNameError('Cannot find a tistory client id in `{}`.'
                                       .format(file_name))

            self.client_id = config[CONFIG_SECTION_NAME][CONFIG_CLIENT_ID]

            # if configuration file includes an account information
            if CONFIG_TISTORY_ID in config[CONFIG_SECTION_NAME]:
                self.tistory_id = config[CONFIG_SECTION_NAME][CONFIG_TISTORY_ID]
            if CONFIG_TISTORY_PASSWORD in config[CONFIG_SECTION_NAME]:
                self.tistory_password = config[CONFIG_SECTION_NAME][CONFIG_TISTORY_PASSWORD]

    # pylint: disable=too-many-arguments
    def configure(self, file_name=None,
                  client_id=None,
                  tistory_id=None,
                  tistory_password=None,
                  force_browser=False,
                  access_token=None):
        """Tistory OAuth 2.0 인증을 실행하는 함수입니다.

        `Tistory Open API OAuth 인증 <http://www.tistory.com/guide/api/oauth>`_ 에
        해당하는 구현이고, Client-side flow방식을 이용하였습니다.
        티스토리 client_id 값을 파일에서 읽거나,
        인자에서 받거나, 환경변수에서 받아서 인증을 하게 됩니다.

        우선순위
         - 함수로 넘어오는 인자값
         - 함수로 넘어오는 파일명에 설정되어 있는 값
         - 환경변수값
         - 기본 파일에 설정되어 있는 값

        만약 file_name도 넘어오고, client_id, tistory_id, tistory_password 중
        하나 이상의 인자가 넘어온다면 file_name에 있던 값들을 인자로 넘긴 값으로 덮어씌운다

        환경 변수의 KEY는 `PYTISTORY_CLIENT_ID` 를
        사용합니다.

        :param file_name: configure 파일 이름, defaults to None
        :type file_name: str, optional
        :param client_id: 티스토리 OAuth를 위한 client_id 값, defaults to None
        :type client_id: str, optional
        :param tistory_id: 티스토리 아이디입니다., defaults to None
        :type tistory_id: str, optional
        :param tistory_password: 티스토리 비밀번호입니다., defaults to None
        :type tistory_password: str, optional
        :param force_browser: 무조건 브라우저를 이용하여 설정합니다., defaults to False
        :type force_browser: bool, optional
        :raises OptionNotFoundError: 인증 과정에서 아무런 인증 옵션이 없을 때 일어나는 에러입니다.
        """
        if access_token is not None:
            self.access_token = access_token
        else:
            try_headless_auth = False

            # with a default credential file
            self._configure_with_file(DEFAULT_CONFIG_FILE_NAME, no_error=True)

            # with env vars
            self.client_id = os.environ.get(
                'PYTISTORY_CLIENT_ID', self.client_id)
            self.tistory_id = os.environ.get(
                'PYTISTORY_TISTORY_ID', self.tistory_id)
            self.tistory_password = os.environ.get(
                'PYTISTORY_TISTORY_PASSWORD', self.tistory_password)

            # with file
            if file_name is not None:
                self._configure_with_file(file_name)

            # with args
            if client_id is not None:
                self.client_id = client_id
            if tistory_id is not None:
                self.tistory_id = tistory_id
            if tistory_password is not None:
                self.tistory_password = tistory_password

            if self.client_id is None:
                raise OptionNotFoundError("Cannot configure a PyTistory.")

            if self.tistory_id is not None and self.tistory_password is not None:
                try_headless_auth = True

            # access token 받아오기
            self._set_access_token(try_headless_auth and not force_browser)

        self.blog.set_access_token(self.access_token)
        self.post.set_access_token(self.access_token)
        self.category.set_access_token(self.access_token)
        self.comment.set_access_token(self.access_token)
        self.guestbook.set_access_token(self.access_token)



# -*- coding: utf8 -*-
"""Post 관련 API Client 구현입니다.
"""
import datetime

from .base_api import BaseAPI


class Post(BaseAPI):
    """Post 관련 API Client 구현입니다.

    다음과 같은 API Client가 구현되어 있습니다.

    - post/list
        최근 게시물 목록을 가져올 수 있는 API입니다.
    - post/write
        게시글을 작성할 수 있는 API입니다.
    - post/modify
        작성된 게시글을 수정할 수 있는 API입니다.
    - post/read
        단일 게시글을 읽을 수 있는 API입니다.
    - post/attach
        파일을 첨부 할 수 있는 API입니다.
    - post/delete
        단일 게시글을 삭제할 수 있는 API입니다.
    """
    # pylint: disable=too-many-arguments
    kind = 'post'

    def list(self, blog_name=None, target_url=None):
        """post/list API 구현입니다.

        최근 게시물 목록을 가져올 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/post.php#post-list>`_ 를 통해
        살펴보실 수 있습니다.

        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :raise TypeError: 인자의 타입이 잘못되었을 때 일어납니다.
        :return:
            `최근 게시글 목록 API <http://www.tistory.com/guide/api/post.php#post-list>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'list')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        response = self._perform('GET', url, params=params)

        return response

    def write(self, title, blog_name=None, target_url=None, visibility=0,
              published=None, category=0, content=None, slogan=None, tag=None):
        """post/list API 구현입니다.

        게시글을 작성할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/post.php#post-write>`_ 를 통해
        살펴보실 수 있습니다.

        :param title: 포스트 제목입니다.
        :type title: str
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :param visibility:
            - 0: 비공개
            - 1: 보호
            - 2: 공개
            - 3: 발행

            defaults to 0
        :type visibility: int, optional
        :param published: 발행 시간. 만약 설정시 예약 발행이 됨., defaults to None
        :type published: :class:`datetime.datetime`, optional
        :param category: 0은 분류없음. 값 설정시 카테고리 설정, defaults to 0
        :type category: int, optional
        :param content: 글 내용, defaults to None
        :type content: str, optional
        :param slogan: 문자 주소. 이는 아마 블로그 주소 형식을 문자로 설정했을 때의 값인 듯 함., defaults to None
        :type slogan: str, optional
        :param tag: 게시글에 태그를 설정합니다, defaults to None
        :type tag: list, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :raise TypeError: 인자의 타입이 잘못되었을 때 일어납니다.
        :return:
            `최근 게시글 목록 API <http://www.tistory.com/guide/api/post.php#post-write>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'write')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        if isinstance(visibility, int) and visibility >= 0 and visibility <= 3:
            params['visibility'] = visibility
        else:
            raise TypeError('A visibility must be 0, 1, 2, or 3.')

        if published:
            if isinstance(published, datetime.datetime):
                params['published'] = published.timestamp()
            else:
                raise TypeError('A published must be a datetime object')

        # dangerous-default-value
        if tag is None:
            tag = []

        if isinstance(tag, list):
            params['tag'] = ','.join(tag)
        else:
            raise TypeError('A tag must be a list.')

        params['title'] = title
        params['category'] = category
        params['content'] = content
        params['slogan'] = slogan

        response = self._perform('POST', url, data=params)

        return response

    def modify(self, title, post_id, blog_name=None, target_url=None, visibility=0,
               category=0, content=None, slogan=None, tag=None):
        """post/modify API 구현입니다.

        작성된 게시글을 수정할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/post.php#post-modify>`_ 를 통해
        살펴보실 수 있습니다.

        :param title: 포스트 제목입니다.
        :type title: str
        :param post_id: 포스트 고유번호입니다.
        :type title: int
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :param visibility:
            - 0: 비공개
            - 1: 보호
            - 2: 공개
            - 3: 발행

            defaults to 0
        :type visibility: int, optional
        :param category: 0은 분류없음. 값 설정시 카테고리 설정, defaults to 0
        :type category: int, optional
        :param content: 글 내용, defaults to None
        :type content: str, optional
        :param slogan: 문자 주소. 이는 아마 블로그 주소 형식을 문자로 설정했을 때의 값인 듯 함., defaults to None
        :type slogan: str, optional
        :param tag: 게시글에 태그를 설정합니다, defaults to None
        :type tag: list, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :raise TypeError: 인자의 타입이 잘못되었을 때 일어납니다.
        :return:
            `최근 게시글 목록 API <http://www.tistory.com/guide/api/post.php#post-modify>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'modify')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        if isinstance(visibility, int) and visibility >= 0 and visibility <= 3:
            params['visibility'] = visibility
        else:
            raise TypeError('A visibility must be 0, 1, 2, or 3.')
        if tag is None:
            tag = []
        if isinstance(tag, list):
            params['tag'] = ','.join(tag)
        else:
            raise TypeError('A tag must be a list.')

        params['title'] = title
        params['postId'] = post_id
        params['category'] = category
        params['content'] = content
        params['slogan'] = slogan

        response = self._perform('POST', url, data=params)

        return response

    def read(self, post_id, blog_name=None, target_url=None):
        """post/read API 구현입니다.

        단일 게시글을 읽을 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/post.php#post-read>`_ 를 통해
        살펴보실 수 있습니다.

        :param post_id: 게시글 번호
        :type post_id: int
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raises NoSpecifiedBlogError: 해당하는 블로그가 존재하지 않을 때 일어나는 에러입니다.
        :return:
            `글 읽기 API  <http://www.tistory.com/guide/api/post.php#post-read>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'read')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        params['postId'] = post_id

        response = self._perform('GET', url, params=params)

        return response

    def attach(self, uploaded_file, blog_name=None, target_url=None):
        """post/attach API 구현입니다.

        파일을 첨부 할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/post.php#post-attach>`_ 를 통해
        살펴보실 수 있습니다.

        :param uploaded_file: 업로드할 파일의 경로입니다.
        :type uploaded_file: str
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raises NoSpecifiedBlogError: 해당하는 블로그가 존재하지 않을 때 일어나는 에러입니다.
        :return:
            `파일 첨부 API  <http://www.tistory.com/guide/api/post.php#post-attach>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'attach')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        with open(uploaded_file, 'rb') as f:
            files = {'uploadedfile': f}
            response = self._perform('POST', url, data=params, files=files)

        return response

    def delete(self, post_id, blog_name=None, target_url=None):
        """post/delete API 구현입니다.

        단일 게시글을 삭제할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/post.php#post-delete>`_ 를 통해
        살펴보실 수 있습니다.

        :param post_id: 삭제할 게시글 번호입니다.
        :type post_id: int
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raises NoSpecifiedBlogError: 해당하는 블로그가 존재하지 않을 때 일어나는 에러입니다.
        :return:
            `글 삭제 API  <http://www.tistory.com/guide/api/post.php#post-delete>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'delete')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        params['postId'] = post_id

        response = self._perform('POST', url, data=params)

        return response



# -*- coding: utf8 -*-
"""Blog 관련 API Client 구현입니다.
"""
from .base_api import BaseAPI


class Blog(BaseAPI):
    """Blog 관련 API Client 구현입니다.

    다음과 같은 API Client가 구현되어 있습니다.

    - blog/info
        사용자의 계정에 해당하는 블로그 정보를 얻어옵니다.
    """
    kind = 'blog'

    def info(self):
        """blog/info API 구현입니다.

        블로그 정보를 받아오는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/blog.php#blog-info>`_ 를 통해
        살펴보실 수 있습니다.

        :return:
            `티스토리 블로그 정보 API <http://www.tistory.com/guide/api/blog.php#blog-info>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'info')
        return self._perform('GET', url, params=self._get_default_params())



# -*- coding:utf8 -*-
#pylint: disable=R0903
# disable the pylint message R0903 `Too few public methods`
"""PyTistory BaseAPI

PyTistory에서 활용하는 API 클래스의 형태를 정의합니다.
"""
import json
import warnings
import requests

from ..exceptions import ParsingError, TokenNotFoundError, NoSpecifiedBlogError, JSONDecodingError


class BaseAPI:
    """다른 API들은 이 클래스를 상속받아 이용합니다.

    공통적으로 쓸 함수들을 포함합니다.
    """
    kind = ''
    access_token = None

    def _set_blog_name(self, params, blog_name, target_url):
        if blog_name:
            params['blogName'] = blog_name
        elif target_url:
            params['targetUrl'] = target_url
            warnings.warn('A parameter `targetUrl` is deprecated.' +
                          ' See also `http://www.tistory.com/guide/api/{0}.php`.'.format(self.kind))
        else:
            raise NoSpecifiedBlogError(
                'There is no blog specified in parameters.')

    def set_access_token(self, access_token):
        """Access Token을 설정합니다.

        :param access_token: Tistory 인증 후 반환되는 Access Token입니다.
        :type access_token: str
        """
        self.access_token = access_token

    def _get_default_params(self):
        """기본적인 인자들을 반환합니다.

        access_token과 output은 기본적인 인자라 함수로 만들어서 dict로 반환하도록 했습니다.
        """
        if self.access_token is None:
            raise TokenNotFoundError('You have not set up an `Access Token` yet.' +
                                     ' Call configure() first.')
        return {
            'access_token': self.access_token,
            'output': 'json'
        }

    @staticmethod
    def _perform(method, url, **kwargs):
        """Tistory API 서버에 요청하고 응답을 받아와주는 함수입니다.

        만약 Status가 200이 아니라면 Exception을 일으키고,
        맞다면 응답 결과를 반환합니다.

        :param method: HTTP Method명을 나타냅니다.
        :type method: str
        :param url: API URL을 나타냅니다.
        :type url: str
        :param params: querystring에 들어갈 인지입니다.
        :type params: dict
        :param data: HTTP Body에 들어갈 인자입니다.
        :type data: dict

        :raises ParsingError: Status를 찾을 수 없거나, 200이 아닐 경우 일어납니다.
        :return: API의 결과값입니다.
        :rtype: dict
        """
        response = requests.request(method, url, **kwargs)
        try:
            result = json.loads(response.text)
        except json.decoder.JSONDecodeError as e:
            raise JSONDecodingError('Cannot parse into json.\nresponse: {}\nerror_message: {}'.format(response.text, e))

        if 'tistory' in result and 'status' in result['tistory']:
            if result['tistory']['status'] == '200':
                return result['tistory']

        raise ParsingError('Status Code is not 200.\nrequest : {} {}, {}\nresponse result: {}'.format(
            method, url, kwargs, result))

    @staticmethod
    def _get_url(kind, action):
        """Tistory API URL을 만드는 함수입니다.

        Kind, Action, Args 인자들을 종합하여 api url을 만들어 반환합니다.
        Kind, Action, Args 값은 Tistory 공식 문서를 참고하시기 바랍니다.

        .. seealso::
            `Tistory 오픈 API 가이드 홈 <http://www.tistory.com/guide/api/index>`_

        :param kind:
            API의 큰 분류라 보면 됩니다. `blog`, `post`, `category`, `comment`,
            `guestbook` 이 있습니다.
        :type kind: str
        :param action:
            API의 작은 분류입니다. 각각 kind 별로 해당하는 값이 존재합니다.
        :type action: str
        """
        return 'https://www.tistory.com/apis/{}/{}'.format(kind, action)



# -*- coding: utf8 -*-
# pylint: disable=W0612
# disable the pylint message W0612 `Unused variable 'callback' and
# 'callback_modified'`
"""티스토리 OAuth 인증을 위한 로컬서버입니다.
"""
from flask import Flask, request
import logging

log = logging.getLogger('werkzeug')
log.setLevel(logging.ERROR)

HASH_TO_ARGS = """
<script>
window.onload = function () {
    location.href = '/callback_modified?' + window.location.hash.substring(1);
}
</script>
"""

CLOSE_WINDOW = """
<script>
window.close()
</script>
"""


class CallbackServer:
    """티스토리 OAuth를 위해 로컬 서버를 엽니다.
    """

    def __init__(self, namespace, event):
        self.namespace = namespace
        self.event = event

    def prepare(self):
        """Flask App을 실행시킵니다.

        `GET /callback`과 `GET /callback_modified`인 두개의 라우팅을 받는데,
        첫번째 라우팅에서는 uri 뒤의 해쉬를 서버에서 파라미터로 받기 위해 다시 고쳐주는 역할만을 합니다.
        두번째 라우팅에서는 access_token을 받아 namespace에 access_token을 설정하고,
        flask app을 종료시킵니다.
        """
        app = Flask(__name__)

        @app.route('/callback')
        def callback():
            """tistory OAuth 인증 redirect_url에 해당하는 함수입니다.

            `/callback_modified`로 리다이렉션 시키는 응답을 내보냅니다.

            :return: 요청 경로 상의 해시를 GET 파라미터로 바꾸어주는 스크립트를 반환합니다.
            :rtype: str
            """
            return HASH_TO_ARGS

        @app.route('/callback_modified')
        def callback_modified():
            """access_token을 실제로 처리하는 함수입니다.

            flask app을 종료시키고, access_token을 메인 프로세스에 전달합니다.

            :return: 창을 닫아주는 스크립트를 반환합니다.
            :rtype: str
            """
            access_token = request.args.get('access_token')
            error = request.args.get('error')
            error_description = request.args.get('error_description')

            # set access token and shutdown server
            self.namespace.access_token = access_token
            self.namespace.error = error
            self.namespace.error_description = error_description
            self.event.set()
            self.shutdown_server()

            return CLOSE_WINDOW

        app.run(host='localhost')

    @staticmethod
    def shutdown_server():
        """flask 앱을 종료시키는 함수입니다.

        :raises RuntimeError: Flask 앱이 실행중이 아니라면 종료할 수 없습니다.
        """
        func = request.environ.get('werkzeug.server.shutdown')
        if func is None:
            raise RuntimeError('Not running with the Werkzeug Server')
        func()



# -*- coding: utf-8 -*-
from codecs import open
from setuptools import setup, find_packages

with open('README.rst', encoding='utf-8') as f:
    README = f.read()

with open('pytistory/__init__.py', encoding='utf-8') as f:
    for line in f.readlines():
        if '__version__' in line:
            version = line.split("'")[1]

setup(
    name='pytistory',
    version=version,
    description="티스토리 블로그 api client입니다.",
    install_requires=[
        'Flask',
        'requests'
    ],
    long_description=README,
    url='https://github.com/JeongUkJae/pytistory',
    author='Jeong Ukjae',
    author_email='jeongukjae@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: End Users/Desktop',
        'Topic :: Internet :: WWW/HTTP :: Site Management',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        'Operating System :: OS Independent',
    ],
    keywords='tistory blogging',
    packages=find_packages(exclude=['tests']),
    test_suite='nose.collector',
    tests_require=['nose']
)



# -*- coding: utf8 -*-
#pylint: disable=too-many-arguments
"""Comment 관련 API Client 구현입니다.
"""
from .base_api import BaseAPI


class Comment(BaseAPI):
    """Comment 관련 API Client 구현입니다.

    다음과 같은 API Client가 구현되어 있습니다.

    - comment/list
        단일 게시글에 포함된 댓글 정보를 조회할 수 있는 API입니다.
    - comment/newest
        블로그내 최근 댓글을 조회할 수 있는 API입니다.
    - comment/write
        단일 게시글 및 단일 댓글에 댓글을 작성할 수 있는 API입니다.
    - comment/modify
        이미 작성된 댓글을 수정할 수 있는 API입니다.
    - comment/delete
        댓글을 삭제할 수 있는 API입니다.
    """
    kind = 'comment'

    def list(self, post_id, blog_name=None, target_url=None):
        """comment/list API 구현입니다.

        단일 게시글에 포함된 댓글 정보를 조회할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/blog.php#comment-list>`_ 를 통해
        살펴보실 수 있습니다.

        :param post_id: 게시글 ID입니다.
        :type post_id: int
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :return:
            `게시글 댓글 목록 API <http://www.tistory.com/guide/api/blog.php#comment-list>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'list')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        params['postId'] = post_id

        return self._perform('GET', url, params=params)

    def newest(self, blog_name=None, target_url=None):
        """comment/newest API 구현입니다.

        블로그내 최근 댓글을 조회할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/blog.php#comment-newest>`_ 를 통해
        살펴보실 수 있습니다.

        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :return:
            `최근 댓글 목록 API <http://www.tistory.com/guide/api/blog.php#comment-newest>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'newest')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        return self._perform('GET', url, params=params)

    def write(self, post_id, content, blog_name=None, target_url=None, parent_id=None, secret=None):
        """comment/write API 구현입니다.

        단일 게시글 및 단일 댓글에 댓글을 작성할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/blog.php#comment-write>`_ 를 통해
        살펴보실 수 있습니다.

        :param post_id: 게시글 ID입니다.
        :type post_id: int
        :param content: 해당 댓글의 내용입니다.
        :type content: str
        :param parent_id: 부모 댓글 id입니다. 댓글의 댓글일 경우만 사용합니다., defaults to None
        :type parent_id: int, optional
        :param secret: 비밀글 여부입니다. 1일 경우 설정됩니다., defaults to None
        :type secret: int, optional
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :return:
            `댓글 작성 API <http://www.tistory.com/guide/api/blog.php#comment-write>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'write')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        params['postId'] = post_id
        params['content'] = content

        if parent_id:
            params['parentId'] = parent_id
        if secret:
            params['secret'] = secret

        return self._perform('POST', url, data=params)

    def modify(self, post_id, comment_id, content, blog_name=None, target_url=None,
               parent_id=None, secret=None):
        """comment/modify API 구현입니다.

        이미 작성된 댓글을 수정할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/blog.php#comment-modify>`_ 를 통해
        살펴보실 수 있습니다.

        :param post_id: 게시글 ID입니다.
        :type post_id: int
        :param comment_id: 수정하려는 댓글 id입니다.
        :type comment_id: int
        :param content: 해당 댓글의 내용입니다.
        :type content: str
        :param parent_id: 부모 댓글 id입니다. 댓글의 댓글일 경우만 사용합니다., defaults to None
        :type parent_id: int, optional
        :param secret: 비밀글 여부입니다. 1일 경우 설정됩니다., defaults to None
        :type secret: int, optional
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :return:
            `댓글 수정 API <http://www.tistory.com/guide/api/blog.php#comment-modify>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'modify')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        params['postId'] = post_id
        params['commentId'] = comment_id
        params['content'] = content

        if parent_id:
            params['parentId'] = parent_id
        if secret:
            params['secret'] = secret

        return self._perform('POST', url, data=params)

    def delete(self, post_id, comment_id, blog_name=None, target_url=None):
        """comment/delete API 구현입니다.

        댓글을 삭제할 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/blog.php#comment-delete>`_ 를 통해
        살펴보실 수 있습니다.

        :param post_id: 게시글 ID입니다.
        :type post_id: int
        :param comment_id: 삭제하려는 댓글 id입니다.
        :type comment_id: int
        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :return:
            `댓글 삭제 API <http://www.tistory.com/guide/api/blog.php#comment-delete>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'delete')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        params['postId'] = post_id
        params['commentId'] = comment_id

        return self._perform('POST', url, data=params)



# -*- coding: utf8 -*-
"""Category 관련 API Client 구현입니다.
"""
from .base_api import BaseAPI


class Category(BaseAPI):
    """Category 관련 API Client 구현입니다.

    다음과 같은 API Client가 구현되어 있습니다.

    - category/list
        카테고리 목록을 가져올 수 있는 API입니다.
    """
    kind = 'category'

    def list(self, blog_name=None, target_url=None):
        """category/list API 구현입니다.

        카테고리 목록을 가져올 수 있는 API입니다. 해당 API에 관한 정보는
        `링크 <http://www.tistory.com/guide/api/blog.php#category-list>`_ 를 통해
        살펴보실 수 있습니다.

        :param blog_name: 블로그 명입니다., defaults to None
        :type blog_name: str, optional
        :param target_url: 블로그의 url입니다. deprecated된 옵션입니다., defaults to None
        :type target_url: str, optional
        :raise NoSpecifiedBlog: 블로그 정보를 설정할 수 없을 때 일어납니다.
        :return:
            `카테고리 목록 API <http://www.tistory.com/guide/api/blog.php#category-list>`_ 링크에서
            어떤 데이터가 넘어오는 지 알 수 있습니다.
        :rtype: dict
        """
        url = self._get_url(self.kind, 'list')
        params = self._get_default_params()
        self._set_blog_name(params, blog_name, target_url)

        return self._perform('GET', url, params=params)



# -*- coding: utf8 -*-
"""PyTistory API

PyTistory의 실질적인 API Client 구현을 담당하는 모듈입니다.
"""
from .blog import Blog
from .post import Post
from .category import Category
from .comment import Comment
from .guestbook import Guestbook



# -*- coding: utf8 -*-
"""pytistory에서 사용되는 Exception들입니다.
"""


class ConfigurationError(Exception):
    """설정이 올바르지 않거나, 설정에 실패했을 때 일어나는 Exception입니다.
    """
    pass


class InvalidSectionError(ConfigurationError):
    """Tistory 인증 정보가 담긴 파일을 파싱하며, 올바른 섹션을 찾을 수 없을 때
    일어나는 Exception입니다.
    """
    pass


class InvalidNameError(ConfigurationError):
    """Tistory 인증 정보가 담긴 파일을 파싱하며, 올바른 Name을 찾을 수 없을 때
    일어나는 Exception입니다.
    """
    pass


class InvalidAccountError(ConfigurationError):
    """Tistory 로그인을 올바르지 않은 계정 정보일 때 일어나는 Exception입니다.
    """
    pass


class EmailAuthError(ConfigurationError):
    """Tistory 로그인을 하면서 이메일 인증이 발생할 때 일어나는 Exception입니다.
    """
    pass


class WebDriverError(ConfigurationError):
    """Tistory 인증 중 Headless Chrome을 사용할 수 없어 일어나는 에러입니다.
    """
    pass


class TokenNotFoundError(ConfigurationError):
    """Tistory 인증 마지막 과정에서 토큰을 찾을 수 없을 때,
    또는 API 실행 중 토큰이 설정되어 있지 않을 때 일어나는 에러입니다.
    """
    pass


class OptionNotFoundError(ConfigurationError):
    """인증과정에서 아무런 설정 옵션을 찾을 수 없을 때 일어나는 에러입니다.
    """
    pass


class ParsingError(Exception):
    """Tistory API를 사용하며 적절한 응답이 오지 않을 때 일어나는 Exception입니다.
    """
    pass

class JSONDecodingError(Exception):
    """Tistory API를 사용할 때 적절한 JSON 문자가 오지 않을 때 일어나는 에러입니다.
    """
    pass

class NoSpecifiedBlogError(Exception):
    """명시된 블로그가 없을때 일어나는 에러입니다.
    """
    pass
